import { Renderer } from "./renderers/renderer";
export declare function compute_renderers<T extends Renderer>(renderers: T[] | "auto" | null, all_renderers: T[]): T[];
//# sourceMappingURL=util.d.ts.map